/*
	LPCUSB, an USB device driver for LPC microcontrollers	
	Copyright (C) 2006 Bertrik Sikken (bertrik@sikken.nl)

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.
	3. The name of the author may not be used to endorse or promote products
	   derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, 
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <string.h>
#include "type.h"
#include "usbdebug.h"

#include "console.h"
#include "usbapi.h"
#include "startup.h"

#include "hwlib/led.h"
#include "utype.h"

#ifdef	LPC214x
#include "hwlib/LPC214x.h"
#else
#include "hwlib/LPC23xx.h"
#endif

#include "monit.h"

extern	Packet PacketFromPC;			//́iw߁jpPbg 64byte
extern	Packet PacketToPC;				//óiԋpjpPbg 64byte

#define	packet_from_pc	PacketFromPC.raw
#define	packet_to_pc	PacketToPC.raw

#define INTR_IN_EP		0x81
#define INTR_OUT_EP		0x01

#define MAX_PACKET_SIZE	64

#define LE_WORD(x)		((x)&0xFF),((x)>>8)

#define REPORT_SIZE		64

uchar wf=0;
uchar lf=0;

void EP1_OUT_Callback(void);

static U8	abClassReqData[64];
//static U8	bufReport[REPORT_SIZE];

static int	_iIdleRate = 0;
//static int	_iFrame = 0;



static U8 abReportDesc[] = 
  {    
    0x06, 0x00, 0xFF,       // Usage Page = 0xFF00 (Vendor Defined Page 1)
    0x09, 0x01,             // Usage (Vendor Usage 1)
    0xA1, 0x01,             // Collection (Application)
    0x19, 0x01,             //      Usage Minimum 
    0x29, 0x40,             //      Usage Maximum 	//64 input usages total (0x01 to 0x40)
    0x15, 0x00,             //      Logical Minimum (data bytes in the report may have minimum value = 0x00)
    0x26, 0xFF, 0x00, 	  	//      Logical Maximum (data bytes in the report may have maximum value = 0x00FF = unsigned 255)
    0x75, 0x08,             //      Report Size: 8-bit field size
    0x95, 0x40,             //      Report Count: Make sixty-four 8-bit fields (the next time the parser hits an "Input", "Output", or "Feature" item)
    0x81, 0x00,             //      Input (Data, Array, Abs): Instantiates input packet fields based on the above report size, count, logical min/max, and usage.
    0x19, 0x01,             //      Usage Minimum 
    0x29, 0x40,             //      Usage Maximum 	//64 output usages total (0x01 to 0x40)
    0x91, 0x00,             //      Output (Data, Array, Abs): Instantiates output packet fields.  Uses same report size and count as "Input" fields, since nothing new/different was specified to the parser since the "Input" item.
    0xC0                   // End Collection
 }; /* CustomHID_ReportDescriptor */

static const U8 abDescriptors[] = {

/* Device descriptor */
	0x12,              		
	DESC_DEVICE,       		
	LE_WORD(0x0110),		// bcdUSB	
	0x00,              		// bDeviceClass
	0x00,              		// bDeviceSubClass
	0x00,              		// bDeviceProtocol
	MAX_PACKET_SIZE0,  		// bMaxPacketSize
//	LE_WORD(0xFFFF),		// idVendor
	LE_WORD(0x0483),		// idVendor
//	LE_WORD(0x0001),		// idProduct
	LE_WORD(0x5750),		// idProduct
	LE_WORD(0x0100),		// bcdDevice
	0x01,              		// iManufacturer
	0x02,              		// iProduct
	0x03,              		// iSerialNumber
	0x01,              		// bNumConfigurations

// configuration
	0x09,
	DESC_CONFIGURATION,
	LE_WORD(0x29),  		// 0x22 wTotalLength
	0x01,  					// bNumInterfaces
	0x01,  					// bConfigurationValue
	0x00,  					// iConfiguration
	0x80,  					// bmAttributes
	0x32,  					// bMaxPower

// interface
	0x09,   				
	DESC_INTERFACE, 
	0x00,  		 			// bInterfaceNumber
	0x00,   				// bAlternateSetting
	0x02,	//0x01,   				// bNumEndPoints
	0x03,   				// bInterfaceClass = HID
	0x00,   				// bInterfaceSubClass
	0x00,   				// bInterfaceProtocol
	0x00,   				// iInterface

// HID descriptor
	0x09, 
	DESC_HID_HID, 			// bDescriptorType = HID
	LE_WORD(0x0110),		// bcdHID
	0x00,   				// bCountryCode
	0x01,   				// bNumDescriptors = report
	DESC_HID_REPORT,   		// bDescriptorType
	LE_WORD(sizeof(abReportDesc)),

// EP descriptor
	0x07,   		
	DESC_ENDPOINT,   		
	INTR_IN_EP,				// bEndpointAddress
	0x03,   				// bmAttributes = INT
	LE_WORD(MAX_PACKET_SIZE),// wMaxPacketSize
	1,	//10,						// bInterval   		

// EP descriptor
	0x07,   		
	DESC_ENDPOINT,   		
	INTR_OUT_EP,				// bEndpointAddress
	0x03,   				// bmAttributes = INT
	LE_WORD(MAX_PACKET_SIZE),// wMaxPacketSize
	1,	//10,						// bInterval   		

// string descriptors
	0x04,
	DESC_STRING,
	LE_WORD(0x0409),

	// manufacturer string
	0x0E,
	DESC_STRING,
    'A', 0, 'V', 0, 'R', 0, 'e', 0, 't', 0, 'c', 0,
//	'L', 0, 'P', 0, 'C', 0, 'U', 0, 'S', 0, 'B', 0,

	// product string
	0x12,
	DESC_STRING,
//	'P', 0, 'r', 0, 'o', 0, 'd', 0, 'u', 0, 'c', 0, 't', 0, 'X', 0,
    'A', 0, 'R', 0, 'M', 0, '3', 0, '2', 0, 's', 0, 'p', 0, 'x', 0,

	// serial number string
	0x12,
	DESC_STRING,
//	'D', 0, 'E', 0, 'A', 0, 'D', 0, 'C', 0, '0', 0, 'D', 0, 'E', 0,
	'0', 0, '0', 0, '0', 0, '0', 0, '0', 0, '0', 0, '0', 0, '1', 0,
	
	// terminator
	0
};


/*************************************************************************
	HandleClassRequest
	==================
		HID class request handler
		
**************************************************************************/
static BOOL HandleClassRequest(TSetupPacket *pSetup, int *piLen, U8 **ppbData)
{
	U8	*pbData = *ppbData;

	switch (pSetup->bRequest) {
	
   	// get_idle
	case HID_GET_IDLE:
		DBG("GET IDLE, val=%X, idx=%X\n", pSetup->wValue, pSetup->wIndex);
		pbData[0] = (_iIdleRate / 4) & 0xFF;
		*piLen = 1;
		break;

	// set_idle:
	case HID_SET_IDLE:
		DBG("SET IDLE, val=%X, idx=%X\n", pSetup->wValue, pSetup->wIndex);
		_iIdleRate = ((pSetup->wValue >> 8) & 0xFF) * 4;
		break;

	default:
		DBG("Unhandled class %X\n", pSetup->bRequest);
		return FALSE;
	}
	return TRUE;
}


#define BAUD_RATE	115200


/*************************************************************************
	HIDHandleStdReq
	===============
		Standard request handler for HID devices.
		
	This function tries to service any HID specific requests.
		
**************************************************************************/
static BOOL HIDHandleStdReq(TSetupPacket *pSetup, int *piLen, U8 **ppbData)
{
	U8	bType, bIndex;

	if ((pSetup->bmRequestType == 0x81) &&			// standard IN request for interface
		(pSetup->bRequest == REQ_GET_DESCRIPTOR)) {	// get descriptor
		
		bType = GET_DESC_TYPE(pSetup->wValue);
		bIndex = GET_DESC_INDEX(pSetup->wValue);
		switch (bType) {

		case DESC_HID_REPORT:
			// report
			*ppbData = abReportDesc;
			*piLen = sizeof(abReportDesc);
			break;

		case DESC_HID_HID:
		case DESC_HID_PHYSICAL:
		default:
		    // search descriptor space
		    return USBGetDescriptor(pSetup->wValue, pSetup->wIndex, piLen, ppbData);
		}
		
		return TRUE;
	}
	return FALSE;
}

//	Device->PC UPLINK
static void _HandleBulkIn(U8 bEP, U8 bEPStatus)
{
	lf=1;LED_Off();
	// 	send next part
	//	USBHwEPWrite(bEP, adrs , size);
}

//	PC->Device DOWNLINK
static void _HandleBulkOut(U8 bEP, U8 bEPStatus)
{
	int iChunk;

//	lf=1;LED_Off();
	// get next part
#if	0
	while(1) {
		iChunk = USBHwEPRead(bEP, packet_from_pc, REPORT_SIZE);
		if(iChunk>0) break;
	}
#else
	iChunk = USBHwEPRead(bEP, packet_from_pc, REPORT_SIZE);
	if(iChunk>0) 
#endif
//	if( wf==0 ) 
	{
		EP1_OUT_Callback();
	}
}

void UserToPMABufferCopy(uint8_t *pbUsrBuf, uint16_t wPMABufAddr, uint16_t wNBytes)
{
	uchar bEP=INTR_IN_EP;
//	static	uchar cnt=0;

	wf=1;
//	pbUsrBuf[8]=0xaa;
//	pbUsrBuf[9]=cnt;cnt++;
	USBHwEPWrite(bEP, pbUsrBuf , REPORT_SIZE);
}
#if	1
static void HandleFrame(U16 wFrame)
{
#if	0
	reply();
//#else
	static int iCount;
	_iFrame++;
	if ((_iFrame > 1000)) {
		// send report (dummy data)
		bufReport[0] = (iCount >> 8) & 0xFF;
		bufReport[1] = (iCount) & 0xFF;
		iCount++;
		USBHwEPWrite(INTR_IN_EP, bufReport, REPORT_SIZE);
		_iFrame = 0;
	}
#endif
}
#endif

uint32_t USB_SIL_Read(uint8_t bEpAddr, uint8_t* pBufferPointer)
{
//	return USBHwEPRead(bEpAddr, pBufferPointer, REPORT_SIZE);
	return 0;
}
uint16_t GetEPTxAddr(uint8_t   bEpNum)
{
	return 0;
}
void     SetEPTxValid(uint8_t bEpNum)
{

}
void 	 SetEPRxStatus(uint8_t bEpNum, uint16_t wState)
{

}
/*************************************************************************
	main
	====
**************************************************************************/
int main(void)
{
	// PLL and MAM
	Initialize();

	// init DBG
	ConsoleInit(60000000 / (16 * BAUD_RATE));

	DBG("Initialising USB stack\n");

	// initialise stack
	USBInit();
	
	// register device descriptors
	USBRegisterDescriptors(abDescriptors);

	// register HID standard request handler
	USBRegisterCustomReqHandler(HIDHandleStdReq);

	// register class request handler
	USBRegisterRequestHandler(REQTYPE_TYPE_CLASS, HandleClassRequest, abClassReqData);

	// register endpoint
	USBHwRegisterEPIntHandler(INTR_IN_EP,  _HandleBulkIn);
	USBHwRegisterEPIntHandler(INTR_OUT_EP, _HandleBulkOut);

	// register frame handler
//	USBHwRegisterFrameHandler(HandleFrame);

	DBG("Starting USB communication\n");

	// connect to bus
	USBHwConnect(TRUE);

	LED_On();

	// call USB interrupt handler continuously
	while (1) {
		USBHwISR();
		USBtask();
#if	APPLICATION_MODE
		if(lf==0) LED_Flip(16);
#endif
	}
	return 0;
}

